@@ -16,15 +16,15 @@ function draw_plot(scene::Scene, screen::Screen, poly::Poly)
     # so, we should also take a look at converted
     # First, we check whether a `draw_poly` method exists for the input arguments
     # before conversion:
-    return if Base.hasmethod(draw_poly, Tuple{Scene, Screen, typeof(poly), typeof.(deref(poly.args[]))...})
-        draw_poly(scene, screen, poly, deref(poly.args[])...)
+    if Base.hasmethod(draw_poly, Tuple{Scene, Screen, typeof(poly), typeof.(deref(poly.args[]))...})
+        return draw_poly(scene, screen, poly, deref(poly.args[])...)
         # If not, we check whether a `draw_poly` method exists for the arguments after conversion
         # (`plot.converted`).  This allows anything which decomposes to be checked for.
     elseif Base.hasmethod(draw_poly, Tuple{Scene, Screen, typeof(poly), typeof.(deref(poly.converted[]))...})
-        draw_poly(scene, screen, poly, deref(poly.converted[])...)
+        return draw_poly(scene, screen, poly, deref(poly.converted[])...)
         # In the worst case, we return to drawing the polygon as a mesh + lines.
     else
-        draw_poly_as_mesh(scene, screen, poly)
+        return draw_poly_as_mesh(scene, screen, poly)
     end
 end
 
@@ -40,30 +40,62 @@ function draw_poly_as_mesh(scene, screen, poly)
     return
 end
 
-# As a general fallback, draw all polys as meshes.
-# This also applies for e.g. per-vertex color.
-function draw_poly(scene::Scene, screen::Screen, poly, points, color, model, strokecolor, strokestyle, strokewidth)
-    return draw_poly_as_mesh(scene, screen, poly)
-end
+########################################
+### outline methods (::Vector{<:VecTypes{2}})
+########################################
 
 function draw_poly(scene::Scene, screen::Screen, poly, points::Vector{<:Point2})
     color = to_cairo_color(poly.color[], poly)
     strokecolor = to_cairo_color(poly.strokecolor[], poly)
     strokestyle = Makie.convert_attribute(poly.linestyle[], key"linestyle"())
-    draw_poly(scene, screen, poly, points, color, poly.model[], strokecolor, strokestyle, poly.strokewidth[])
-    return if color isa Cairo.CairoPattern
+
+    miter_limit = to_cairo_miter_limit(poly.miter_limit[])
+    joinstyle = to_cairo_joinstyle(poly.joinstyle[])
+    linecap = to_cairo_linecap(poly.linecap[])
+
+    draw_poly(
+        scene, screen, poly, points, color,
+        poly.model[], strokecolor, strokestyle, poly.strokewidth[], miter_limit, joinstyle, linecap
+    )
+    if color isa Cairo.CairoPattern
         pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
     end
+    return
 end
 
+function draw_poly(scene::Scene, screen::Screen, poly, points_list::Vector{<:Vector{<:Point2}})
+    color = to_cairo_color(poly.color[], poly)
+    strokecolor = to_cairo_color(poly.strokecolor[], poly)
+    strokestyle = Makie.convert_attribute(poly.linestyle[], key"linestyle"())
+
+    miter_limit = to_cairo_miter_limit(poly.miter_limit[])
+    joinstyle = to_cairo_joinstyle(poly.joinstyle[])
+    linecap = to_cairo_linecap(poly.linecap[])
+
+    broadcast_foreach(
+        points_list, color,
+        strokecolor, strokestyle, poly.strokewidth[], Ref(poly.model[])
+    ) do points, color, strokecolor, strokestyle, strokewidth, model
+        draw_poly(scene, screen, poly, points, color, model, strokecolor, strokestyle, strokewidth, miter_limit, joinstyle, linecap)
+    end
+    if color isa Cairo.CairoPattern
+        pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
+    end
+    return
+end
+
+draw_poly(scene::Scene, screen::Screen, poly, circle::Circle) = draw_poly(scene, screen, poly, decompose(Point2f, circle))
+
 # when color is a Makie.AbstractPattern, we don't need to go to Mesh
 function draw_poly(
         scene::Scene, screen::Screen, poly, points::Vector{<:Point2}, color::Union{Colorant, Cairo.CairoPattern},
-        model, strokecolor, strokestyle, strokewidth
+        model, strokecolor, strokestyle, strokewidth, miter_limit, joinstyle, linecap
     )
     space = poly.space[]
+    points = apply_transform(transform_func(poly), points)
     points = clip_poly(poly.clip_planes[], points, space, model)
     points = _project_position(scene, space, points, model, true)
+
     Cairo.move_to(screen.context, points[1]...)
     for p in points[2:end]
         Cairo.line_to(screen.context, p...)
@@ -75,26 +107,31 @@ function draw_poly(
     Cairo.fill_preserve(screen.context)
     Cairo.set_source_rgba(screen.context, rgbatuple(to_color(strokecolor))...)
     Cairo.set_line_width(screen.context, strokewidth)
-    isnothing(strokestyle) || Cairo.set_dash(screen.context, diff(Float64.(strokestyle)) .* strokewidth)
-    return Cairo.stroke(screen.context)
+    set_miter_limit(screen.context, miter_limit)
+    Cairo.set_line_join(screen.context, joinstyle)
+    Cairo.set_line_cap(screen.context, linecap)
+
+    pattern = to_cairo_linestyle(strokestyle, strokewidth)
+    if !isnothing(pattern)
+        Cairo.set_dash(screen.context, pattern)
+    end
+    Cairo.stroke(screen.context)
+    return
 end
 
-function draw_poly(scene::Scene, screen::Screen, poly, points_list::Vector{<:Vector{<:Point2}})
-    color = to_cairo_color(poly.color[], poly)
-    strokecolor = to_cairo_color(poly.strokecolor[], poly)
-    strokestyle = Makie.convert_attribute(poly.linestyle[], key"linestyle"())
-
-    broadcast_foreach(
-        points_list, color,
-        strokecolor, strokestyle, poly.strokewidth[], Ref(poly.model[])
-    ) do points, color, strokecolor, strokestyle, strokewidth, model
-        draw_poly(scene, screen, poly, points, color, model, strokecolor, strokestyle, strokewidth)
-    end
-    return if color isa Cairo.CairoPattern
-        pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
-    end
+# As a general fallback, draw all polys as meshes.
+# This also applies for e.g. per-vertex color.
+function draw_poly(
+        scene::Scene, screen::Screen, poly, points, color,
+        model, strokecolor, strokestyle, strokewidth, miter_limit, joinstyle, linecap
+    )
+    return draw_poly_as_mesh(scene, screen, poly)
 end
 
+########################################
+### GeometryPrimtive and BezierPath methods
+########################################
+
 draw_poly(scene::Scene, screen::Screen, poly, rect::Rect2) = draw_poly(scene, screen, poly, [rect])
 draw_poly(scene::Scene, screen::Screen, poly, bezierpath::BezierPath) = draw_poly(scene, screen, poly, [bezierpath])
 
@@ -119,18 +156,30 @@ function draw_poly(scene::Scene, screen::Screen, poly, shapes::Vector{<:Union{Re
         error("Wrong type for linestyle: $(poly.linestyle[]).")
     end
     strokecolor = to_cairo_color(poly.strokecolor[], poly)
+
+    miter_limit = to_cairo_miter_limit(poly.miter_limit[])
+    joinstyle = to_cairo_joinstyle(poly.joinstyle[])
+    linecap = to_cairo_linecap(poly.linecap[])
+
     broadcast_foreach(projected_shapes, color, strokecolor, poly.strokewidth[]) do shape, c, sc, sw
         create_shape_path!(screen.context, shape)
         set_source(screen.context, c)
         Cairo.fill_preserve(screen.context)
-        isnothing(linestyle_diffed) || Cairo.set_dash(screen.context, linestyle_diffed .* sw)
+        pattern = to_cairo_linestyle(linestyle, sw)
+        if !isnothing(pattern)
+            Cairo.set_dash(screen.context, pattern)
+        end
         set_source(screen.context, sc)
         Cairo.set_line_width(screen.context, sw)
+        set_miter_limit(screen.context, miter_limit)
+        Cairo.set_line_join(screen.context, joinstyle)
+        Cairo.set_line_cap(screen.context, linecap)
         Cairo.stroke(screen.context)
     end
-    return if color isa Cairo.CairoPattern
+    if color isa Cairo.CairoPattern
         pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
     end
+    return
 end
 
 function project_shape(scene, space, shape::BezierPath, model)
@@ -153,9 +202,19 @@ function create_shape_path!(ctx, r::Rect2)
 end
 
 function create_shape_path!(ctx, b::BezierPath)
+    isempty(b.commands) && return
+    if !isa(first(b.commands), MoveTo)
+        error("The first command in a BezierPath must be `MoveTo`, but is $(first(b.commands)).")
+    end
+
     for cmd in b.commands
         path_command(ctx, cmd)
     end
+
+    # match W/GLMakie
+    if !isa(last(b.commands), ClosePath)
+        Cairo.close_path(ctx)
+    end
     return
 end
 
@@ -184,7 +243,6 @@ end
 
 draw_poly(scene::Scene, screen::Screen, poly, polygon::Polygon) = draw_poly(scene, screen, poly, [polygon])
 draw_poly(scene::Scene, screen::Screen, poly, multipolygon::MultiPolygon) = draw_poly(scene, screen, poly, multipolygon.polygons)
-draw_poly(scene::Scene, screen::Screen, poly, circle::Circle) = draw_poly(scene, screen, poly, decompose(Point2f, circle))
 
 function draw_poly(scene::Scene, screen::Screen, poly, polygons::AbstractArray{<:Polygon})
     model = poly.model[]
@@ -197,18 +255,30 @@ function draw_poly(scene::Scene, screen::Screen, poly, polygons::AbstractArray{<
     strokecolor = to_cairo_color(poly.strokecolor[], poly)
     strokestyle = Makie.convert_attribute(poly.linestyle[], key"linestyle"())
 
-    broadcast_foreach(projected_polys, color, strokecolor, strokestyle, poly.strokewidth[]) do po, c, sc, ss, sw
+    miter_limit = to_cairo_miter_limit(poly.miter_limit[])
+    joinstyle = to_cairo_joinstyle(poly.joinstyle[])
+    linecap = to_cairo_linecap(poly.linecap[])
+
+    broadcast_foreach(projected_polys, color, strokecolor, poly.strokewidth[]) do po, c, sc, sw
         polypath(screen.context, po)
         set_source(screen.context, c)
         Cairo.fill_preserve(screen.context)
         set_source(screen.context, sc)
         Cairo.set_line_width(screen.context, sw)
+        pattern = to_cairo_linestyle(strokestyle, sw)
+        if !isnothing(pattern)
+            Cairo.set_dash(screen.context, pattern)
+        end
+        set_miter_limit(screen.context, miter_limit)
+        Cairo.set_line_join(screen.context, joinstyle)
+        Cairo.set_line_cap(screen.context, linecap)
         Cairo.stroke(screen.context)
     end
 
-    return if color isa Cairo.CairoPattern
+    if color isa Cairo.CairoPattern
         pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
     end
+    return
 end
 
 function draw_poly(scene::Scene, screen::Screen, poly, polygons::AbstractArray{<:MultiPolygon})
@@ -221,21 +291,32 @@ function draw_poly(scene::Scene, screen::Screen, poly, polygons::AbstractArray{<
     color = to_cairo_color(poly.color[], poly)
     strokecolor = to_cairo_color(poly.strokecolor[], poly)
     strokestyle = Makie.convert_attribute(poly.linestyle[], key"linestyle"())
-    broadcast_foreach(projected_polys, color, strokecolor, strokestyle, poly.strokewidth[]) do mpo, c, sc, ss, sw
+
+    miter_limit = to_cairo_miter_limit(poly.miter_limit[])
+    joinstyle = to_cairo_joinstyle(poly.joinstyle[])
+    linecap = to_cairo_linecap(poly.linecap[])
+    broadcast_foreach(projected_polys, color, strokecolor, poly.strokewidth[]) do mpo, c, sc, sw
         for po in mpo.polygons
             polypath(screen.context, po)
             set_source(screen.context, c)
             Cairo.fill_preserve(screen.context)
             set_source(screen.context, sc)
-            isnothing(ss) || Cairo.set_dash(screen.context, diff(Float64.(ss)) .* sw)
             Cairo.set_line_width(screen.context, sw)
+            pattern = to_cairo_linestyle(strokestyle, sw)
+            if !isnothing(pattern)
+                Cairo.set_dash(screen.context, pattern)
+            end
+            set_miter_limit(screen.context, miter_limit)
+            Cairo.set_line_join(screen.context, joinstyle)
+            Cairo.set_line_cap(screen.context, linecap)
             Cairo.stroke(screen.context)
         end
     end
 
-    return if color isa Cairo.CairoPattern
+    if color isa Cairo.CairoPattern
         pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
     end
+    return
 end
 
 
@@ -325,6 +406,11 @@ function draw_plot(
         if color isa Cairo.CairoPattern
             pattern_set_matrix(color, Cairo.CairoMatrix(1, 0, 0, 1, 0, 0))
         end
+
+        for p in band.plots
+            p isa Mesh && continue
+            draw_plot(scene, screen, p)
+        end
     else
         for p in band.plots
             draw_plot(scene, screen, p)
