@@ -33,7 +33,7 @@ function build_combined_transformation_matrix(
         scene::Scene, space::Symbol, model::Mat4, yflip = true
     )
     f32convert = Makie.f32_convert_matrix(scene.float32convert, space)
-    M = Makie.space_to_clip(scene.camera, space) * f32convert * model
+    M = Makie.get_space_to_space_matrix(scene, space, :clip) * f32convert * model
     return cairo_viewport_matrix(scene.camera.resolution[], yflip) * M
 end
 
@@ -109,12 +109,15 @@ end
 
 
 function project_shape(@nospecialize(scenelike), space, rect::Rect, model)
-    mini = project_position(scenelike, space, minimum(rect), model)
-    maxi = project_position(scenelike, space, maximum(rect), model)
+    # Note that this assumes the Rect to remain axis-aligned through transformations
+    # clamp to prevent float issues from switching to widths = maxi .- mini
+    res = Makie.get_scene(scenelike).camera.resolution[]
+    mini = clamp.(project_position(scenelike, space, minimum(rect), model), -res, 2 .* res)
+    maxi = clamp.(project_position(scenelike, space, maximum(rect), model), -res, 2 .* res)
     return Rect(mini, maxi .- mini)
 end
 
-function clip_poly(clip_planes::Vector{Plane3f}, ps::Vector{PT}, space::Symbol, model::Mat4) where {PT <: VecTypes{2}}
+function clip_poly(clip_planes::Vector{Plane3f}, ps::AbstractVector{PT}, space::Symbol, model::Mat4) where {PT <: VecTypes{2}}
     if isempty(clip_planes) || !Makie.is_data_space(space)
         return ps
     end
@@ -150,7 +153,7 @@ function clip_shape(clip_planes::Vector{Plane3f}, shape::Rect2, space::Symbol, m
 
     xy = origin(shape)
     w, h = widths(shape)
-    ps = Vec2f[xy, xy + Vec2f(w, 0), xy + Vec2f(w, h), xy + Vec2f(0, h)]
+    ps = Point2d[xy, xy + Vec2d(w, 0), xy + Vec2d(w, h), xy + Vec2d(0, h)]
     if any(p -> Makie.is_clipped(clip_planes, p), ps)
         push!(ps, xy)
         ps = clip_poly(clip_planes, ps, space, model)
@@ -296,6 +299,60 @@ function set_source(ctx::Cairo.CairoContext, color::Colorant)
     return Cairo.set_source_rgba(ctx, rgbatuple(color)...)
 end
 
+#######################################
+#        Stroking properties          #
+#######################################
+
+function to_cairo_linecap(linecap_symb::Symbol)
+    linecap = Makie.convert_attribute(linecap_symb, key"linecap"())
+    return to_cairo_linecap(linecap)
+end
+function to_cairo_linecap(linecap)
+    if linecap == 1
+        return Cairo.CAIRO_LINE_CAP_SQUARE
+    elseif linecap == 2
+        return Cairo.CAIRO_LINE_CAP_ROUND
+    elseif linecap == 0
+        return Cairo.CAIRO_LINE_CAP_BUTT
+    else
+        error("$linecap is not a valid linecap. Valid: 0 (:butt), 1 (:square), 2 (:round)")
+    end
+end
+
+function to_cairo_joinstyle(joinstyle_symb::Symbol)
+    joinstyle = Makie.convert_attribute(joinstyle_symb, key"joinstyle"())
+    return to_cairo_joinstyle(joinstyle)
+end
+function to_cairo_joinstyle(joinstyle)
+    if joinstyle == 2
+        return Cairo.CAIRO_LINE_JOIN_ROUND
+    elseif joinstyle == 3
+        return Cairo.CAIRO_LINE_JOIN_BEVEL
+    elseif joinstyle == 0
+        return Cairo.CAIRO_LINE_JOIN_MITER
+    else
+        error("$joinstyle is not a valid linecap. Valid: 0 (:miter), 2 (:round), 3 (:bevel)")
+    end
+end
+
+function to_cairo_miter_limit(miter_limit)
+    return 2.0f0 * Makie.miter_angle_to_distance(miter_limit)
+end
+
+to_cairo_linestyle(::Nothing, ::Any) = nothing
+to_cairo_linestyle(::AbstractVector, ::AbstractArray) = nothing
+function to_cairo_linestyle(linestyle::AbstractVector, linewidth::Real)
+    # There is a discrepancy between Makie and Cairo when it comes to linestyles.
+    # For Makie, the linestyle array is cumulative, and defines the "absolute"
+    # endpoints of segments. However, for Cairo, each value provides the length of
+    # alternate "on" and "off" portions of the stroke. Therefore, we take the
+    # diff of the given linestyle, to convert the "absolute" coordinates into
+    # "relative" ones.
+    pattern = diff(Float64.(linestyle)) .* linewidth
+    isodd(length(pattern)) && push!(pattern, 0)
+    return pattern
+end
+
 ########################################
 #        Marker conversion API         #
 ########################################
@@ -316,10 +373,10 @@ cairo_scatter_marker(marker) = Makie.to_spritemarker(marker)
 
 to_cairo_image(img::AbstractMatrix{<:Colorant}) = to_cairo_image(to_uint32_color.(img))
 
-function to_cairo_image(img::Matrix{UInt32})
+function to_cairo_image(img::AbstractMatrix{UInt32})
     # we need to convert from column-major to row-major storage,
     # therefore we permute x and y
-    return Cairo.CairoARGBSurface(permutedims(img))
+    return Cairo.CairoARGBSurface(convert(Matrix, permutedims(img)))
 end
 
 
